/*=================================================================
===								===
=== FILE	: xgkstest.c					===
===								===
=== CONTENTS	: Example program to demonstrate Xgks widget.	===
===		  Creates an XGKS workstation, and three	===
===		  command widgets:				===
===			square		: draw a square		===
===			triangle	: draw a triangle	===
===			quit		: no prizes		===
===								===
=================================================================*/

#include <stdio.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
#include <X11/Xmu/Converters.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include "XgksWidget.h"
#include <X11/xgks/xgks.h>

#define WS_ID		1
#define TRANSFORM	1

#define RED		128
#define GREEN		129
#define BLUE		130

void		CreateWidgets(),
		QuitProc(),
		SquareCallback(),
		TriangleCallback();

Gpoint		triangle_pts[4] = {{20,20},{60,20},{40,60},{20,20}};
Gpoint		square_pts[5] = {{20,20},{60,20},{60,60},{20,60},{20,20}};
Gcobundl	color_defs[3] = {{0.8,0.0,0.0},{0.0,0.8,0.0},{0.0,0.0,0.8}};


void
main(argc,argv)
	unsigned int	argc;
	char		*argv[];
{
	XtAppContext	app_con;
	Widget		toplevel,label,form,box,xgks_widget,
			square,triangle,quit;
	XEvent		event;
	int		ok_to_continue = 1;		/* gag! */


	/* Initialize XGKS */
	if (gopengks(stdout,0) != 0)
                XtError("XgksError: gopengks failed");

	toplevel = XtAppInitialize(&app_con, "xgkstest", NULL, 0, &argc, argv,
				NULL,NULL,0);

	CreateWidgets(toplevel, &ok_to_continue);

	XtRealizeWidget(toplevel);

	if (gactivatews(WS_ID) != 0)
                XtError("XgksError: activate_ws failed");


	/* define the colors red, green and blue */
	gsetcolorrep(WS_ID,RED,&color_defs[0]);
	gsetcolorrep(WS_ID,GREEN,&color_defs[1]);
	gsetcolorrep(WS_ID,BLUE,&color_defs[2]);

	InitGksTrans(WS_ID);

	/* Replacement for XtAppMainLoop, since we want to fall thru */
	while (ok_to_continue)
	{
		XtAppNextEvent(app_con,&event);
		XtDispatchEvent(&event);
	}
	XtDestroyWidget(toplevel);
	XtDestroyApplicationContext(app_con);
	gclosegks();
}


/*--------------------------------------------------------
-- CreateWidgets()					--
--	Creates the following widget hierarchy under	--
--	the widget passed:				--
--		Form  --+-- XgksWidget			--
--			|				--
--			+-- Box --+-- Label		--
--				  +-- Square		--
--				  +-- Triangle		--
--				  +-- Quit		--
--------------------------------------------------------*/
void
CreateWidgets(toplevel, flag)
	Widget		toplevel;
	int		*flag;
{
	Widget		label,form,box,xgks_widget,
			square,triangle,quit;


	form = XtCreateManagedWidget("form",formWidgetClass,toplevel,
					NULL,0);

	/* Box widget for label and buttons */
	box = XtCreateManagedWidget("box",boxWidgetClass,form,
					NULL,0);
	XtVaSetValues(box,
		XtNorientation,	(XtArgVal) XtorientHorizontal,
		XtNleft,	(XtArgVal) XtChainLeft,
		XtNright,	(XtArgVal) XtChainLeft,
		XtNtop,		(XtArgVal) XtChainTop,
		XtNbottom,	(XtArgVal) XtChainTop,
		XtNborderWidth, (XtArgVal) 0,
		NULL);

	label = XtCreateManagedWidget("label",labelWidgetClass,box,NULL,0);
	XtVaSetValues(label,
		XtNlabel,	(XtArgVal)"XgksWidget",
		XtNborderWidth,	(XtArgVal)0,
		NULL);

	square = XtCreateManagedWidget("square",commandWidgetClass,box,NULL,0);
	XtVaSetValues(square,XtNlabel,(XtArgVal)"Square",NULL);
	XtAddCallback(square,XtNcallback,SquareCallback,NULL);

	triangle = XtCreateManagedWidget("triangle",commandWidgetClass,box,
						NULL,0);
	XtVaSetValues(triangle,XtNlabel,(XtArgVal)"Triangle",NULL);
	XtAddCallback(triangle,XtNcallback,TriangleCallback,NULL);

	quit = XtCreateManagedWidget("quit",commandWidgetClass,box,NULL,0);
	XtVaSetValues(quit,XtNlabel,(XtArgVal)"Quit",NULL);
	XtAddCallback(quit,XtNcallback,QuitProc,(XtPointer)flag);

	xgks_widget = XtCreateManagedWidget("xgks1",xgksWidgetClass,form,
						NULL,0);
	XtVaSetValues(xgks_widget,
		XtNwsId,	(XtArgVal) WS_ID,
		XtNwidth,	(XtArgVal) 300,
		XtNheight,	(XtArgVal) 240,
		XtNleft,	(XtArgVal) XtChainLeft,
		XtNright,	(XtArgVal) XtChainRight,
		XtNtop,		(XtArgVal) XtChainTop,
		XtNbottom,	(XtArgVal) XtChainBottom,
		XtNfromVert,	(XtArgVal) box,
		NULL);
}


/*--------------------------------------------------------
-- InitGksTrans()					--
--	Defines the normalization transformation, and	--
--	initializes the locator.			--
--------------------------------------------------------*/
InitGksTrans(ws_id)
	int		ws_id;
{
	static Glimit	w = {0.0, 99.0, 0.0, 79.0},
			v = {0.0, 1.0, 0.0, 0.8},
			wsw = {0.0, 1.0, 0.0, 0.8},
			wsv = {0.0, 1279.0, 0.0, 1023.0};
	Gloc		loc_init;
	Glocrec		loc_rec;


	gsetwindow(TRANSFORM, &w);
	gsetviewport(TRANSFORM, &v);
	gselntran(TRANSFORM);
	gsetwswindow(ws_id, &wsw);
	gsetwsviewport(ws_id, &wsv);
	loc_init.transform = TRANSFORM;
	loc_init.position.x = loc_init.position.y = 0;
	loc_rec.pet1.data = NULL;
	ginitloc(ws_id,1,&loc_init,1,&wsv,&loc_rec);
}


/*--------------------------------------------------------
-- QuitProc()						--
--	Callback for quit button -- deactivates the	--
--	workstation and sets flag to die in main().	--
--------------------------------------------------------*/
void
QuitProc(widget,closure,call_data)
	Widget		widget;
	caddr_t		closure;
	caddr_t		call_data;
{
	int		*ok2cont	= (int *) closure;


	*ok2cont = 0;
	gdeactivatews(WS_ID);
}


/*--------------------------------------------------------
-- TriangleCallback()					--
--	Callback for triangle button -- clears the	--
--	workstation and draws a green triangle.		--
--------------------------------------------------------*/
void
SquareCallback(widget,closure,call_data)
	Widget		widget;
	caddr_t		closure;
	caddr_t		call_data;
{
	gclearws(WS_ID,GALWAYS);
	gsetlinecolorind(RED);
	gpolyline(5, square_pts);
}


/*--------------------------------------------------------
-- TriangleCallback()					--
--	Callback for triangle button -- clears the	--
--	workstation and draws a green triangle.		--
--------------------------------------------------------*/
void
TriangleCallback(widget,closure,call_data)
	Widget		widget;
	caddr_t		closure;
	caddr_t		call_data;
{
	gclearws(WS_ID,GALWAYS);
	gsetlinecolorind(GREEN);
	gpolyline(4, triangle_pts);
}
